; $Id: kred.scm,v 1.4 1993/06/14 07:03:20 czyborra Exp $

(define (draw circle) (apply kred-draw-circle circle))

(define (show circles) ; zeichnet alle Kreise in der Liste circles
  (for-each draw (reverse circles))
  (kred-show-drawing))

(define (delete what where)
  (if (null? where) '()
      (if (equal? what (car where)) (cdr where)
	  (cons (car where) (delete what (cdr where))))))

(define (drag circles) ; wartet auf MB-release und liefert Position
  (let* ((event (kred-get-next-event))
	 (evtype (car event)))
    (if (eq? evtype 'expose) 
	(begin
	  (show circles)
	  (drag circles))
	(let* ((evchar (cadr event))
	       (evpoxy (cddr event)))
	  (if (eq? evtype 'button-release)
	      evpoxy
	      (drag circles))))))

(define (distance A B)
  (ceiling (sqrt (apply + (map (lambda (x) (* x x)) (map - A B))))))

(define maxdist 100000)

(define (nearest circles P)
  ; liefert die Nhe des naheliegendsten Kreises zu P und seine Koordinaten
  (if (null? circles) (list maxdist)
      (let* ((rest (nearest (cdr circles) P))
	     (first (car circles))
	     (center (list (car first) (cadr first)))
	     (radius (caddr first))
	     (proximity (abs (- radius (distance center P)))))
	(if (< proximity (car rest))
	    (cons proximity first)
	    rest))))

(define (save circles)
  (let ((port (open-output-file "kred.farbkreise")))
    (write circles port)
    (close-output-port port)))

(define (kred-loop circles latest show?) ; Hauptschleife
  (if show? (show circles))
  (let* 
      ((event (kred-get-next-event))
       (evtype (car event)))
    (case evtype
      ((expose)       (kred-loop circles latest #t))
      ((vanish) (save circles) (exit))
      ((key-press button-press) 
       (let ((evchar (cadr event))
	     (evpoxy (cddr event)))
       (case evtype
         ((key-press)
          (case (if (= 1 (string-length evchar)) (string-ref evchar 0) #\0)
            ((#\u) (if (null? latest) (kred-loop circles '() #f)
                       (if (member latest circles)
                           (kred-loop (delete latest circles) latest #t)
                           (kred-loop (cons latest circles) latest #t))))
            ((#\w) (save circles)
                   (kred-loop circles latest #f))
            ((#\r) (let* ((port (open-input-file "kred.farbkreise"))
                          (data (read port)))
                     (close-output-port port)
                     (kred-loop data '() #t)))
            ((#\q) (exit))
            (else (kred-loop circles latest #f))))
         ((button-press)
          (case evchar
            ((1 2) (let* ((radius (distance evpoxy (drag circles)))
                        (circle (append evpoxy (list radius evchar))))
                   (kred-loop (cons circle circles) circle #t)))
            ((3) (let ((latest (cdr (nearest circles evpoxy))))
                   (kred-loop (delete latest circles) latest #t)))
            (else (kred-loop circles latest #f)))))))
      (else (kred-loop circles latest #f)))))

(if (kred-init) (kred-loop '() '() #f))
